//==================================================================
//    Project:    IncBldNm.   This project is designed to increment
//                a constant variable in a source code module.  The
//                variable is intended to represent a 'build' number
//                and can be run prior to building the project.
//
//    Module:    IncBldNm.c 
//    Author:    Jim Gentilin
//    Date:      June 2000
//==================================================================


#include <windows.h>
#include <windowsx.h>
#include <stdio.h>
#include <memory.h>
#include <string.h>
#include <direct.h>

//-- This programs version
#define    MAJOR_VER    1    // X.--
#define    MINOR_VER    0    // -.X-
#define REVISION_VER    0    // -.-X

//--  forward ref prototypes
void banner(void);    //-- Simple screen message
void syntax(void);    //-- program usage

//==================================================================
// main()  Application Entry Point
//    we are supplied the program command line arguments.  Valid 
//    arguments include:
//    argc[1] = source file to modify (required)
//    argc[2] = variable name to increment its value (optional)
//            Default variable name ="BuildNumber")
//==================================================================

int main(int argc, char* argv[])
{

    char FilePath[MAX_PATH];       //-- will hold the name of the 
                                   //-- file to modify
    char BuildNumber[MAX_PATH];    //-- search string 
                                   //-- i.e. MyProjectBuildNumber 

    int retcode = 0;               //-- assume a good outcome
    unsigned char found = FALSE;   //-- assume no find
    LPSTR pos;
    LPSTR psuf;
    UINT num_digits;
    char * newnum;
    UINT i;

    banner();    //-- show the startup banner

    //-- no params, or the '?' switch, show the program syntax
    if( argc <= 1 || argv[1][1] == '?' ) 
    {
        syntax();
        return(2);
    }
    else
    {
        HANDLE hFile, hFileMap;    //-- memory mapped file handle
        LPVOID lpvFile;            //-- memory file pointer
        DWORD dwFileSize;            //-- file size

        //-- build the fully qualified path if only a file 
        //--name is supplied
        if( strchr( argv[1], '\\' ) == NULL )
        {   //-- no folder separators found, make a path
            //-- using the current directory
            getcwd( FilePath,MAX_PATH);
            strcat( FilePath, "\\" );
            strcat( FilePath, argv[1] );
        }
        else
            strcpy( FilePath, argv[1] );

        //-- build the search string
        if( argc >= 3 )
        {   //-- search string was supplied
            strcpy(BuildNumber, argv[2] );
        }
        else            //-- use the default "BuildNumber"
            strcpy( BuildNumber, "BuildNumber" );

        //-- Open the source file for read/write
        hFile = CreateFile(FilePath,
                    GENERIC_WRITE | GENERIC_READ,
                    0, NULL,
                    OPEN_EXISTING,
                    FILE_ATTRIBUTE_NORMAL,
                    NULL);

        if (hFile == INVALID_HANDLE_VALUE) 
        {   //-- File open failed.
            printf("File could not be opened.\r\n");
            return(2);
        }

        //-- Get the file size 
        dwFileSize = GetFileSize(hFile, NULL);

        //-- Create the file mapping object. The file-mapping
        //-- object is 2 characters larger than the file size.
        //-- One byte for the terminating zero (NULL) so that 
        //-- we can use stdlib string and copy routines.  The 
        //-- other byte is in case we need to lengthen the 
        //-- file because the version number increased the 
        //-- number of decimal places by one.

        hFileMap = CreateFileMapping(hFile, NULL,
                    PAGE_READWRITE,
                    0, 
                    dwFileSize +( 2 * sizeof(char)),
                     NULL);

        if (hFileMap == NULL) 
        {
            //-- File mapping failed.
            printf("File map could not be opened.\r\n");
            CloseHandle(hFile);
            return(2);
        }

        //-- Create a Map view to get the address of where the file
        //-- is mapped into memory.
        lpvFile = MapViewOfFile(hFileMap,
                        FILE_MAP_WRITE, 0, 0, 0);

        if (lpvFile == NULL) 
        {
            //-- Map view failed.
            printf("Could not map view of file.\r\n");
            CloseHandle(hFileMap);
            CloseHandle(hFile);
            return(2);
        }

        //-- Put a zero character at the very end of the file.
        ((LPSTR)lpvFile)[dwFileSize / sizeof(char)] = 0;

        //-- find the "VersionNumber" sub string
        pos = strstr((const char *)lpvFile, BuildNumber );
        if( pos != NULL )
        {   //-- found the search string!
            //-- assign a pointer to the memory location
            //-- just past the variable name 
            LPSTR pstr = pos + strlen( BuildNumber ); 
            //-- start searching for the first number 
            //-- characer representing the build number

            //-- loop until we replace our build number or
            //-- we encounter the end of the file
            while( !found && 
                pstr < (char *)lpvFile + dwFileSize) 
            {
                if( *pstr >= '0' && *pstr <= '9' )
                {   //-- convert the string to a number
                    DWORD num = (DWORD)atol(pstr);
                    //-- increment the build number 
                    num++;
                    //-- get the number suffix by 
                    //-- searching anything other than a and ASCII
                    //-- '0' through '9'
                    psuf = pstr;
                   //-- loop until we replace our build number or
                   //-- we encounter the end of the file
                    while( !found && 
                            psuf < (char *)lpvFile + dwFileSize)
                    {
                        if( *psuf >= '0' && *psuf <= '9' )
                            ++psuf;    //-- its still a number...
                        else
                        {   //-- ok, here we need to back up psuf 
                            //-- by one to ref the least significant
                            //-- digit of the build number
                            --psuf;
                            //-- ok, now we need to see if the new 
                            //-- number is going to be longer than 
                            //-- the old.  If so, we need to do a 
                            //-- memmove() on the remaining file
                            //-- to make room for the extra digit
                            num_digits = (UINT)
                                ((LPSTR)psuf-(LPSTR)pstr)+1;
                            newnum = (char *)malloc( num_digits+2);
                            sprintf( newnum,"%d",num);

                            if( (int)strlen(newnum) > 
                                (int)(num_digits))
                            {   //-- new is longer, shift the 
                                //-- remaining data to make room
                                //--  for the extra digit.
                                //-- memmove() is safe for 
                                //-- overlapping data blocks.
                              memmove(pstr+1, pstr,
                                  dwFileSize - 
                                 ( pstr-(char *)lpvFile )+1);
                                //-- increment the filesize.
                                //-- to account for the change
                                ++dwFileSize;
                            }
                            else
                            {   //-- test for a smaller new number
                                //-- ( old number had zero padded 
                                //-- prefix )
                                if( (int)strlen(newnum) < 
                                    (int)(num_digits))
                                {   //-- build the formatting string
                                    char numformatter[5];
                                    sprintf( numformatter, "%%0%dd",
                                        num_digits );
                                    //-- re-do the new number with
                                    //-- prefixed zeros
                                    sprintf( newnum,
                                        numformatter,num);
                                }
                            }
                            //-- substitue the new data for the old
                            //-- by simply overwriting it
                            for( i = 0; i< strlen(newnum); i++)
                            {
                                *pstr++ = newnum[i];
                            }
                            free(newnum);
                            printf( 
                                " changed variable '%s' to %d\r\n",
                                BuildNumber, num );
                            found = TRUE; //-- done!
                        }
                    }
                }
                ++pstr;
            }
        } //-- end of if( pos != NULL )
        else
        {
            printf( " couldn't find variable named '%s'\r\n",
                    BuildNumber );
            retcode = 4;
        }

        //-- Clean up the memory mapped file view and handle
        UnmapViewOfFile(lpvFile);
        CloseHandle(hFileMap);

        //-- Adjust the file size (if necessary).  This will remove
        //-- the extra byte for the 'null' character.
        SetFilePointer(hFile, dwFileSize, NULL, FILE_BEGIN);
        SetEndOfFile(hFile);

        CloseHandle(hFile);
    }

    return (retcode);
}

//==================================================================
//--  banner
//==================================================================
void banner( void )
{
    char version[10];
    sprintf( version, "%d.%d%d",
        MAJOR_VER, MINOR_VER, REVISION_VER);
    printf( "IncBuildNum (v%s) ", version );
}

//==================================================================
//--  syntax
//==================================================================
void syntax( void )
{
    printf( "increments the 'BuildNumber' varaible located \
in a text file\r\n");
    printf( "  typical BuildNum.h file have the line: \
const WORD BuildNumber = 1; \r\n" );
    printf( "  You must supply a filename as an argument \
to this program.\r\n");
    printf( "\r\nIncBuildNum MyHeader.h [MyProgramBuildNumber]\
\r\n\r\n" );
}
