// myxfrp.c - Entrypoints for Performance Data Collection

#define UNICODE
#define _UNICODE

#include <windows.h>
#include <tchar.h>
#include "myxfrp.h"
#include "myxfrnm.h"

PM_OPEN_PROC    OpenMyPerfPerformanceData;
PM_COLLECT_PROC CollectMyPerfPerformanceData;
PM_CLOSE_PROC   CloseMyPerfPerformanceData;

DWORD dwCount = 0;
HANDLE hSyncEvent = NULL;
MYXFR_DATA_DEFINITION MyXfrData;
extern MYXFR_SHARED_DATA shrMem;    // shared buffer

//-----------------------------------------------------------------
DWORD APIENTRY OpenMyXfrPerformanceData(LPWSTR lpDeviceNames)
{
    HKEY hKey = NULL;
    DWORD dwSize, dwType, dwCnt1 = 0, dwHelp1 = 0, status;

    // Create name syncronization event

    hSyncEvent = CreateEvent(NULL, FALSE, TRUE, MYXFR_SYNC_NAME);

    if (!dwCount) {

        // retrieve counter and help index base values

        if ((status = RegOpenKeyEx(HKEY_LOCAL_MACHINE,
                                   MYXFR_REGKEY, 0L, KEY_READ, 
                                   &hKey)) != ERROR_SUCCESS) {
            return status;
        }

        dwSize = sizeof(dwSize);
        RegQueryValueEx(hKey, TEXT("First Counter"), 0L, &dwType, 
                        (LPBYTE)&dwCnt1, &dwSize);

        dwSize = sizeof(dwSize);
        RegQueryValueEx(hKey, TEXT("First Help"), 0L, &dwType, 
                        (LPBYTE)&dwHelp1, &dwSize);

        RegCloseKey(hKey);
        memset(&MyXfrData, 0, sizeof(MyXfrData));

        // init PERF_OBJECT_TYPE structure

        MyXfrData.MyXfrObjectType.TotalByteLength = 
            sizeof(MYXFR_DATA_DEFINITION) + sizeof(MYXFR_COUNTER);
        MyXfrData.MyXfrObjectType.DefinitionLength = 
            sizeof(MYXFR_DATA_DEFINITION);
        MyXfrData.MyXfrObjectType.HeaderLength = 
            sizeof(PERF_OBJECT_TYPE);
        MyXfrData.MyXfrObjectType.ObjectNameTitleIndex = 
            MYXFR_OBJ + dwCnt1;
        MyXfrData.MyXfrObjectType.ObjectHelpTitleIndex = 
            MYXFR_OBJ + dwHelp1;
        MyXfrData.MyXfrObjectType.DetailLevel = PERF_DETAIL_NOVICE;
        MyXfrData.MyXfrObjectType.NumCounters = MYXFR_COUNTERS;
        MyXfrData.MyXfrObjectType.DefaultCounter = 0;
        MyXfrData.MyXfrObjectType.NumInstances = MYXFR_INSTANCES;

        // init PERF_COUNTER_DEFINITION block for each counter

        MyXfrData.BytesReadDef.ByteLength = 
                    sizeof(PERF_COUNTER_DEFINITION);
        MyXfrData.BytesReadDef.CounterNameTitleIndex = 
                    MYXFR_BYTESREAD + dwCnt1;
        MyXfrData.BytesReadDef.CounterHelpTitleIndex = 
                    MYXFR_BYTESREAD + dwHelp1;
        MyXfrData.BytesReadDef.DetailLevel = PERF_DETAIL_NOVICE;
        MyXfrData.BytesReadDef.CounterType = PERF_COUNTER_RAWCOUNT;
        MyXfrData.BytesReadDef.CounterSize = sizeof(DWORD);
        MyXfrData.BytesReadDef.CounterOffset = 
                    (DWORD_PTR)&(((PMYXFR_COUNTER)0)->BytesRead);

        MyXfrData.BytesWrittenDef.ByteLength = 
                    sizeof(PERF_COUNTER_DEFINITION);
        MyXfrData.BytesWrittenDef.CounterNameTitleIndex = 
                    MYXFR_BYTESWRITTEN + dwCnt1;
        MyXfrData.BytesWrittenDef.CounterHelpTitleIndex = 
                    MYXFR_BYTESWRITTEN + dwHelp1;
        MyXfrData.BytesWrittenDef.DetailLevel = PERF_DETAIL_NOVICE;
        MyXfrData.BytesWrittenDef.CounterType = 
                    PERF_COUNTER_RAWCOUNT;
        MyXfrData.BytesWrittenDef.CounterSize = sizeof(DWORD);
        MyXfrData.BytesWrittenDef.CounterOffset = 
                    (DWORD_PTR)&(((PMYXFR_COUNTER)0)->BytesWritten);

        MyXfrData.BytesTotalDef.ByteLength = 
                    sizeof(PERF_COUNTER_DEFINITION);
        MyXfrData.BytesTotalDef.CounterNameTitleIndex = 
                    MYXFR_BYTESTOTAL + dwCnt1;
        MyXfrData.BytesTotalDef.CounterHelpTitleIndex = 
                    MYXFR_BYTESTOTAL + dwHelp1;
        MyXfrData.BytesTotalDef.DetailLevel = PERF_DETAIL_NOVICE;
        MyXfrData.BytesTotalDef.CounterType = PERF_COUNTER_RAWCOUNT;
        MyXfrData.BytesTotalDef.CounterSize = sizeof(DWORD);
        MyXfrData.BytesTotalDef.CounterOffset = 
                    (DWORD_PTR)&(((PMYXFR_COUNTER)0)->BytesTotal);

        MyXfrData.ReadCallsDef.ByteLength = 
                    sizeof(PERF_COUNTER_DEFINITION);
        MyXfrData.ReadCallsDef.CounterNameTitleIndex = 
                    MYXFR_READCALLS + dwCnt1;
        MyXfrData.ReadCallsDef.CounterHelpTitleIndex = 
                    MYXFR_READCALLS + dwHelp1;
        MyXfrData.ReadCallsDef.DetailLevel = PERF_DETAIL_NOVICE;
        MyXfrData.ReadCallsDef.CounterType = PERF_COUNTER_RAWCOUNT;
        MyXfrData.ReadCallsDef.CounterSize = sizeof(DWORD);
        MyXfrData.ReadCallsDef.CounterOffset = 
                    (DWORD_PTR)&(((PMYXFR_COUNTER)0)->ReadCalls);

        MyXfrData.WriteCallsDef.ByteLength = 
                    sizeof(PERF_COUNTER_DEFINITION);
        MyXfrData.WriteCallsDef.CounterNameTitleIndex = 
                    MYXFR_WRITECALLS + dwCnt1;
        MyXfrData.WriteCallsDef.CounterHelpTitleIndex = 
                    MYXFR_WRITECALLS + dwHelp1;
        MyXfrData.WriteCallsDef.DetailLevel = PERF_DETAIL_NOVICE;
        MyXfrData.WriteCallsDef.CounterType = PERF_COUNTER_RAWCOUNT;
        MyXfrData.WriteCallsDef.CounterSize = sizeof(DWORD);
        MyXfrData.WriteCallsDef.CounterOffset = 
                    (DWORD_PTR)&(((PMYXFR_COUNTER)0)->WriteCalls);

        MyXfrData.TotalCallsDef.ByteLength = 
                    sizeof(PERF_COUNTER_DEFINITION);
        MyXfrData.TotalCallsDef.CounterNameTitleIndex = 
                    MYXFR_TOTALCALLS + dwCnt1;
        MyXfrData.TotalCallsDef.CounterHelpTitleIndex = 
                    MYXFR_TOTALCALLS + dwHelp1;
        MyXfrData.TotalCallsDef.DetailLevel = PERF_DETAIL_NOVICE;
        MyXfrData.TotalCallsDef.CounterType = PERF_COUNTER_RAWCOUNT;
        MyXfrData.TotalCallsDef.CounterSize = sizeof(DWORD);
        MyXfrData.TotalCallsDef.CounterOffset = 
                    (DWORD_PTR)&(((PMYXFR_COUNTER)0)->TotalCalls);
    }

    dwCount++; 
    return ERROR_SUCCESS;

} // OpenMyXfrPerformanceData

//-----------------------------------------------------------------
DWORD APIENTRY CollectMyXfrPerformanceData(
    IN      LPWSTR  lpwszValueName,
    IN OUT  LPVOID  *lppData,
    IN OUT  LPDWORD lpcbBytes,
    IN OUT  LPDWORD lpcObjectTypes)
{
    PMYXFR_DATA_DEFINITION pMyXfrData;
    PERF_INSTANCE_DEFINITION *pInstance;
    PMYXFR_COUNTER pCounter;
    DWORD queryType = 0, size;
    LPWSTR pName = NULL, pszToken = NULL;
    TCHAR szValueName[MAX_PATH];
    BOOL bFound = FALSE;

    // validate parameters

    *lpcObjectTypes = 0;

    if (*lpcbBytes < (sizeof(MYXFR_DATA_DEFINITION) +  
                      sizeof(PERF_INSTANCE_DEFINITION) + 
                      sizeof(MYXFR_INSTANCE_NAME) +
                      sizeof(MYXFR_COUNTER))) { 
        *lpcbBytes = 0;
        return ERROR_MORE_DATA;
    }
    *lpcbBytes = 0;

    if (lpwszValueName && *lpwszValueName) {
        lstrcpy(szValueName, lpwszValueName);
        pszToken = wcstok(szValueName, TEXT(" "));
        if (pszToken) {
            if (lstrcmpi(pszToken, GLOBAL_STR) == 0) {
                ;                       // all non-costly counters
            } else if (lstrcmpi(pszToken, COSTLY_STR) == 0) {
                return ERROR_SUCCESS;   // no costly counters
            } else if (lstrcmpi(pszToken, FOREIGN_STR) == 0) {
                return ERROR_SUCCESS;   // no foreign computers
            } else {                    // must be index(s)
                while (pszToken) {
                    if ((DWORD)_ttol(pszToken) == MyXfrData.
                            MyXfrObjectType.ObjectNameTitleIndex) {
                        bFound = TRUE;
                        break;
                    }
                    pszToken = wcstok(szValueName, TEXT(" "));
                }
                if (!bFound) {
                    return ERROR_SUCCESS;   // not supported
                }
            }
        }
    }

    // initialize with static values

    pMyXfrData = (PMYXFR_DATA_DEFINITION)*lppData;
    memmove(pMyXfrData, &MyXfrData, sizeof(MYXFR_DATA_DEFINITION));

    //  fill in dynamic counter data

    size = (lstrlen(MYXFR_INSTANCE_NAME) + 1) * sizeof(WCHAR);
    pInstance = (PERF_INSTANCE_DEFINITION *)&pMyXfrData[1];
    pInstance->ByteLength = sizeof(PERF_INSTANCE_DEFINITION) +
                            DWORD_MULTIPLE(size);

    pInstance->ParentObjectTitleIndex = 0;
    pInstance->ParentObjectInstance = 0;
    pInstance->UniqueID = -1;
    pInstance->NameOffset = sizeof(PERF_INSTANCE_DEFINITION);
    pInstance->NameLength = size;

    pName = (LPWSTR)&pInstance[1];
    memcpy(pName, MYXFR_INSTANCE_NAME, size);
    pCounter = (PVOID)((PCHAR)pInstance + pInstance->ByteLength);

    WaitForSingleObject(hSyncEvent, 5000);

    pCounter->CounterBlock.ByteLength = sizeof(MYXFR_COUNTER);
    pCounter->BytesRead    = shrMem.BytesRead;
    pCounter->BytesWritten = shrMem.BytesWritten;
    pCounter->ReadCalls    = shrMem.ReadCalls;
    pCounter->WriteCalls   = shrMem.WriteCalls;
    pCounter->BytesTotal   = shrMem.BytesRead + shrMem.BytesWritten;
    pCounter->TotalCalls   = shrMem.ReadCalls + shrMem.WriteCalls;

    SetEvent(hSyncEvent);

    *lpcObjectTypes = 1;        
    pInstance = (PERF_INSTANCE_DEFINITION *)&pCounter[1];
    *lppData = (PVOID)pInstance;
    *lpcbBytes = (DWORD)((PBYTE)pInstance - (PBYTE)pMyXfrData);
    pMyXfrData->MyXfrObjectType.TotalByteLength = *lpcbBytes;
    pMyXfrData->MyXfrObjectType.NumInstances = MYXFR_INSTANCES;

    return ERROR_SUCCESS;

} // CollectMyXfrPerformanceData

//-----------------------------------------------------------------
DWORD APIENTRY CloseMyXfrPerformanceData(void)
{
    if (!(--dwCount)) {     // last thread
        WaitForSingleObject(hSyncEvent, 5000);
        CloseHandle(hSyncEvent);
    }
    return ERROR_SUCCESS;

} // CloseMyXfrPerformanceData 
